<?php declare(strict_types=1);

namespace App\Tests\Unit\Entity;

use App\Entity\FoodOrder;
use App\Entity\FoodVendor;
use App\Entity\MenuItem;
use InvalidArgumentException;
use Symfony\Component\Uid\Ulid;

use function describe;
use function test;

describe(FoodVendor::class, function (): void {
    test('FoodVendor entity', function (): void {
        $vendor = new FoodVendor;
        $vendor->setName('Test');
        $this->assertEquals('Test', $vendor->getName());
        $this->assertInstanceOf(Ulid::class, $vendor->getId());
        $this->assertEmpty($vendor->getPhone());
        $vendor->setPhone('1234567890');
        $this->assertEquals('1234567890', $vendor->getPhone());

        // Test emojis field
        $this->assertNull($vendor->getEmojis());
        $emojis = '😀😂🎉👍❤️';
        $vendor->setEmojis($emojis);
        $this->assertEquals($emojis, $vendor->getEmojis());

        // Test emojis validation
        $tooManyEmojis = '😀😂🎉👍❤️🚀🎈🎁🎊🎋🎍🎎🎏🎐🎑🎒🎓🎔🎕🎖🎗🎘🎙🎚🎛🎜🎝🎞🎟🎠🎡🎢';
        $this->expectException(InvalidArgumentException::class);
        $vendor->setEmojis($tooManyEmojis);

        $this->assertCount(0, $vendor->getFoodOrders());
        $order1 = new FoodOrder;
        $vendor->addFoodOrder($order1);
        $vendor->addFoodOrder($order1);
        $this->assertCount(1, $vendor->getFoodOrders());
        $this->assertSame($vendor, $order1->getFoodVendor());
        $vendor->removeFoodOrder($order1);
        $this->assertCount(0, $vendor->getFoodOrders());
        $this->assertNull($order1->getFoodVendor());
    });

    test('MenutItem entity', function (): void {
        $vendor = new FoodVendor;
        $menuItem1 = new MenuItem;
        $menuItem2 = new MenuItem;
        $this->assertCount(0, $vendor->getMenuItems());
        $vendor->addMenuItem($menuItem1);
        $vendor->addMenuItem($menuItem1);
        $this->assertCount(1, $vendor->getMenuItems());
        $vendor->removeMenuItem($menuItem1);
        $this->assertCount(0, $vendor->getMenuItems());
        $this->assertNull($menuItem1->getFoodVendor());
        $vendor->addMenuItem($menuItem1);
        $menuItem2->delete();
        $vendor->addMenuItem($menuItem2);
        $this->assertCount(1, $vendor->getMenuItems());
        $this->assertCount(2, $vendor->getMenuItems(true));
    });

    test('remove foreign menu item', function (): void {
        $vendor1 = new FoodVendor;
        $vendor2 = new FoodVendor;
        $item1 = new MenuItem;

        $vendor1->addMenuItem($item1);
        $this->assertCount(1, $vendor1->getMenuItems());
        $vendor2->removeMenuItem($item1);
        $this->assertCount(1, $vendor1->getMenuItems());
        $this->assertSame($vendor1, $item1->getFoodVendor());
    });

})->covers(FoodVendor::class, FoodOrder::class, MenuItem::class);
